/**
 *******************************************************************************
 * @file    main.c
 *
 * @brief   Main program body.
 *
 * @author  DPM
 *
 * @version V1.0.0
 *
 * @date    2024-5-6
 *
 *******************************************************************************/

/*
    Copyright (c) 2024, DP Microelectronics Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

/* Files include */
#include "main.h"
#include "dpm32m0xx.h"
#include "dpm32m0xx_eval.h"
#include "systick.h"
#include <stdio.h>

/**
  * @addtogroup DPM32M08X_LibSamples
  * @{
  */

/**
  * @addtogroup I2C
  * @{
  */

/**
  * @addtogroup I2C_SlaveInterrupt
  * @{
  */

/* Private typedef --------------------------------------------------------------*/
I2C_InitTypeStruct I2C_InitStruct;

/* Private define ---------------------------------------------------------------*/

/* Private macro ----------------------------------------------------------------*/

/* Private variables ------------------------------------------------------------*/
uint8_t  int_flag_cnt_rxne = 0;
uint8_t  int_flag_cnt_txe = 0;
uint8_t  int_flag_cnt_stop = 0;
uint8_t  int_flag_cnt_start = 0;
uint8_t  int_flag_cnt_match = 0;
uint8_t tx_buf[5] = {0x11, 0x22, 0x44, 0x88, 0xAA};
uint8_t rx_buf[5] ;

/* Private functions ------------------------------------------------------------*/

/**
  * @brief  The program entry function.
  * @retval int
  */
int main(void)
{
  /* Configure the system clock */
  SystemClockInit();
  /* Configure the system tick */
  SystickInit();
  /* LED Init */
  EVAL_LedInit(LED1);

  /* Print Uart Init */
  EVAL_ComInit();

  /********************* Operation  *********************/
  /* Enable clock */
  RCC_Unlock();
  RCC_AHBPeriphClockCmd(RCC_AHB_PERIPH_GPIO, ENABLE);
  RCC_APBPeriphClockCmd(RCC_APB_PERIPH_I2C, ENABLE);
  RCC_Lock();
  /* Cofig I2C GPIO */
  GPIO_InitTypeStruct GPIO_InitStruct;
  GPIO_InitStruct.GPIO_Mode = GPIO_MODE_ALT_FUNC;
  GPIO_InitStruct.GPIO_Speed = GPIO_SPEED_HIGH;
  GPIO_InitStruct.GPIO_OType = GPIO_OTYPE_OD;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PUPD_NOPULL;
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_4 | GPIO_PIN_5;
  GPIO_Init(GPIOD, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOD, GPIO_PIN_4 | GPIO_PIN_5, GPIO_ALT_FUNC_1);
  I2C_DeInit();
  /* Config I2C */
  I2C_StructInit(&I2C_InitStruct);
  I2C_InitStruct.I2C_OwnAddress = 0x00;
  I2C_InitStruct.I2C_Prescaler = 0x16;
  I2C_InitStruct.I2C_Timing = 0x00420F13;
  I2C_InitStruct.I2C_DigitalFilter = 0x0F;
  I2C_InitStruct.I2C_Mode = I2C_MODE_SLAVE;
  I2C_InitStruct.I2C_AddressingMode = I2C_ADDRESSING_MODE_7BIT;
  I2C_Init(&I2C_InitStruct);
  /* Enable I2C */
  I2C_Cmd(ENABLE);
  /* Enable interrupt */
  NVIC_SetPriority(I2C_IRQn, 2);
  NVIC_EnableIRQ(I2C_IRQn);
  
  uint32_t time = GetTick() + 1000;

  /* I2C transmit */
  printf("********************* I2C Transmit  *********************\n");
  I2C_IntCmd(I2C_INT_TYPE_TXE, ENABLE);
  I2C_IntCmd(I2C_INT_TYPE_ADDRM, ENABLE);
  I2C_IntCmd(I2C_INT_TYPE_STOPF, ENABLE);
  I2C_IntCmd(I2C_INT_TYPE_START, ENABLE);

  for(uint16_t i = 0; i < 5; i++)
  {
    printf("tx_buf[%d] = 0X%x\n", i, tx_buf[i]);
  }

  int_flag_cnt_rxne = 0;
  int_flag_cnt_txe = 0;
  int_flag_cnt_stop = 0;
  int_flag_cnt_start = 0;
  int_flag_cnt_match = 0;

  /* Wait address match */
  while((int_flag_cnt_match < 1 || int_flag_cnt_start == 0) && (time > GetTick()));

  /* Send 5 byte data */
  while(int_flag_cnt_txe < 5 && (time > GetTick()));

  /* Wait stop */
  while(int_flag_cnt_stop == 0 && (time > GetTick()));

  /* I2C receive */
  printf("********************* I2C Receive  *********************\n");
  time = GetTick() + 1000;
  I2C_IntCmd(I2C_INT_TYPE_RXNE, ENABLE);
  I2C_IntCmd(I2C_INT_TYPE_ADDRM, ENABLE);
  I2C_IntCmd(I2C_INT_TYPE_STOPF, ENABLE);
  I2C_IntCmd(I2C_INT_TYPE_START, ENABLE);

  int_flag_cnt_rxne = 0;
  int_flag_cnt_txe = 0;
  int_flag_cnt_stop = 0;
  int_flag_cnt_start = 0;
  int_flag_cnt_match = 0;

  /* Wait address match */
  while((int_flag_cnt_match == 0 || int_flag_cnt_start == 0) && (time > GetTick()));

  /* Receive 5 byte data */
  while(int_flag_cnt_rxne < 5 && (time > GetTick()));

  /* Wait stop */
  while(int_flag_cnt_stop == 0 && (time > GetTick()));

  for(uint16_t i = 0; i < 5; i++)
  {
    printf("rx_buf[%d] = 0X%x\n", i, rx_buf[i]);
  }

  uint32_t ledTime = 0;
  /* Infinite loop */
  while(1)
  {
    /* Runing led toggle */
    if(ledTime < GetTick())
    {
      ledTime = GetTick() + 1000;
      EVAL_LedToggle(LED1);
    }
  }
}
/* retarget the C library printf function to the usart */
int fputc(int ch, FILE *f)
{
  UART_SendData(EVAL_COM0, (uint8_t)ch);

  while(RESET == UART_GetIntFlagStatus(EVAL_COM0, UART_INT_FLAG_TXE));

  return ch;
}




/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT DP *****END OF FILE ***************************/
